@@ -5,6 +5,20 @@ import typing
 import h5py
 
 
+@dataclasses.dataclass
+class GRP:
+    """Group names for h5md files."""
+
+    edges: str = "edges"
+    boundary: str = "boundary"
+    position: str = "position"
+    energy: str = "energy"
+    species: str = "species"
+    forces: str = "forces"
+    stress: str = "stress"
+    velocity: str = "velocity"
+
+
 @dataclasses.dataclass
 class FormatHandler:
     filename: str
@@ -47,6 +61,11 @@ class FormatHandler:
             ) from err
 
     @property
-    def box(self):
-        group = f"particles/{self.particle_key}/box/edges"
+    def edges(self):
+        group = f"particles/{self.particle_key}/box/{GRP.edges}"
+        return self.file[group]
+
+    @property
+    def boundary(self):
+        group = f"particles/{self.particle_key}/box/{GRP.boundary}"
         return self.file[group]
